@@ -79,7 +79,8 @@ func (t *Terminal) Start(execCmd []string, getOutput bool, wait bool, callback f
 	if getOutput {
 		t.output = bytes.NewBuffer([]byte{})
 	}
-	Term, _, err := terminal.Start(&t.State, cmd, t.output)
+	cmd.Stdout = t.output
+	Term, _, err := terminal.Start(&t.State, cmd)
 	if err != nil {
 		return err
 	}
@@ -128,7 +129,13 @@ func (t *Terminal) Close() {
 	// call the lua function that the user has given as a callback
 	if t.getOutput {
 		if t.callback != nil {
-			t.callback(t.output.String())
+			Jobs <- JobFunction{
+				Function: func(out string, args []interface{}) {
+					t.callback(out)
+				},
+				Output: t.output.String(),
+				Args:   nil,
+			}
 		}
 	}
 }
